/**
 * Module:  firmware
 * Version: 1v2
 * Build:   6ee030bb6889ef7fa8bd871765d4954233229644
 * File:    src/common/softReset/appSoftReset.h
 *
 * The copyrights, all other intellectual and industrial 
 * property rights are retained by XMOS and/or its licensors. 
 * Terms and conditions covering the use of this code can
 * be found in the Xmos End User License Agreement.
 *
 * Copyright XMOS Ltd 2009
 **/                                   

// Reset uses a hard-coded address in kernel stack space to 
// determine how the system boots after reset.
// The word consists of the following:
//  byte[3:2]  Magic value.
//  byte[1]    Boot program ID
//  byte[0]    Boot program ID (duplicated)

// If the magic value is invalid, or the two boot IDs dont match, 
// the system will boot from SPI (and then the XDK.xe image on SD)
// If the magic value is valid, and hte two boot IDs match, the following
// boot modes exist:

// Boot program ID
// 0xFF  - JTAG
// 0x00  - first image on the file system
// 0x01  - second image on the file system
// etc.

#ifndef _APP_SOFT_RESET_H
#define _APP_SOFT_RESET_H

#define SOFT_RESET_MAGIC_VALUE 0xABCD0000
#define SOFT_RESET_INVALID_VALUE 0x12345678
#define SOFT_RESET_BOOT_JTAG 0xFF
#define SOFT_RESET_BOOT_SPI  0xFE

void _SetNextAppNum(unsigned int num);
unsigned int _GetNextAppNum();
void _InvalidateAppNum();

#endif

