/**
 * Module:  firmware
 * Version: 1v2
 * Build:   6ee030bb6889ef7fa8bd871765d4954233229644
 * File:    src/firmware.xc
 *
 * The copyrights, all other intellectual and industrial 
 * property rights are retained by XMOS and/or its licensors. 
 * Terms and conditions covering the use of this code can
 * be found in the Xmos End User License Agreement.
 *
 * Copyright XMOS Ltd 2009
 **/                                   
// System headers
#include <xs1.h>

#include <platform.h>
#include "sw_comps_common.h"
#include "appSoftReset.h"
#include <print.h>

#include "firmware.h"
#include "bootConfig.h"

// Demo headers
#include "clock.h"
#include "audio.h"
#include "reaction.h"
#include "uart.h"

extern void flashLed(out port p, unsigned onPulse, unsigned offPulse, unsigned noFlashes, unsigned onVal);
unsigned doKeys(chanend c, unsigned killMask, in port p_key);

void setRegVal(int psctl, int reg, int procc, int n);

/* Port declarations */
// Keys/Key-leds
out port p_kled = PORT_BUTTONLED;	
in port p_key	= PORT_BUTTON; 	

// 'Clock' leds
out port p_cled_g = PORT_CLOCKLED_SELG;
out port p_cled_r = PORT_CLOCKLED_SELR;
out port p_cled_0 = PORT_CLOCKLED_0;
out port p_cled_1 = PORT_CLOCKLED_1;
out port p_cled_2 = PORT_CLOCKLED_2;

buffered out port:32 p_spk = XS1_PORT_1K;

#define DEMO_FLASHS 5            // Number of times to flash key LED on demo selection
#define DEMO_KILL_MASK 1

// Call Demos from these functions
// NOTE: EACH DEMO SHOULD TAKE A CHANEND AS PARAM, AND DIE ON RECEIVE CMD_KILL
// Demos can also receive key values from this channel (CMD_KEYVAL) command, which is sent
// everytime there is a change in button state.
void runDemo(int demoNumber, in port p_key)
{
  chan c;
  unsigned tmp;

  flashLed(p_kled, 10000000, 10000000, DEMO_FLASHS, (1<<demoNumber));
  par
  {
    tmp = doKeys(c, DEMO_KILL_MASK, p_key);
    {  
      if(demoNumber ==0)
        runClock(c);    
      else if(demoNumber == 1) 
        runAudio(c);
      else if(demoNumber == 2)
        runReaction(c);
      else if(demoNumber == 3)
        runUart(c);
    }
  }
}


//Simple delay using passed timer
void wait(timer tmr, unsigned delay)
{
  unsigned s;
  tmr :> s;
  tmr when timerafter(s+delay) :> unsigned t ;
}


void SetBootSticky(unsigned bootMagicNo)
{
  unsigned bootMagicNum = SOFT_RESET_MAGIC_VALUE | (bootMagicNo << 8) | bootMagicNo;
  _SetNextAppNum(bootMagicNum);
}


#define TIMESTEP 800000    //time it takes to turn the led on or off
#define RESOLUTION 6000     //the number of time slices per time step
#define INCREMENTS 50     //the amount of brightness step
#define GREENTIME_STEP 150

void outClockLeds(int x)
{
  p_cled_0 <: x;
  p_cled_1 <: x;
  p_cled_2 <: x; 
}

void ledFade(int selVal, int greenTime, int cLedValr)
{
  timer t;
  int i, j;
  unsigned const step = (TIMESTEP/RESOLUTION)/INCREMENTS;

  // LED fade on
  for(i=1;i<INCREMENTS ;i++)
  {
    p_cled_r <: selVal;
    p_cled_g <: !selVal;
    for(int j = 0; j <= greenTime;j++)
    {  
      outClockLeds(cLedValr); 
      wait(t, i*step);
      outClockLeds(~cLedValr);
      wait(t, (INCREMENTS-i)*step); 
    }
       
    p_cled_g <: selVal;
    p_cled_r <: !selVal;
    for(j = 0; j < RESOLUTION-greenTime;j++)
    {  
      outClockLeds(~cLedValr);
      wait(t, i*step);
      outClockLeds(cLedValr);
      wait(t, (INCREMENTS-i)*step); 
    } 
  }
}


// Does something pretty on the 'clock' LEDs, untill killed by user press
void  doClockLedGlow(chanend c, chanend c1)
{
  int loopTerm = 1;
  int cLedValr = 0xa;
  timer t;
  unsigned time, tmp, selVal = 1;
  unsigned greenTime = GREENTIME_STEP;

  p_cled_r <: 1;
  p_cled_g <: 0;

  while(loopTerm)
  {
    t :> time;
    select
    {
      case c :> tmp:
        if(tmp == CMD_KILL)
        {
          p_cled_r <: 0;
          p_cled_g <: 0;
          p_cled_0 <: 0;
          p_cled_1 <: 0;
          p_cled_2 <: 0;
          loopTerm = 0;
 
          c1 <: CMD_KILL;     // Kill keyled thread
        }
        break;
   
      case t when timerafter(time+1000) :> int tmpTime: 
        greenTime+=GREENTIME_STEP;
        if(greenTime>=(RESOLUTION-GREENTIME_STEP))
        {
          selVal = !selVal;
          greenTime = GREENTIME_STEP;
        }
        //LED fade On
        ledFade(selVal, greenTime, cLedValr);
        ledFade(selVal, greenTime, ~cLedValr); 
        break;
    }
  }
}

void fadeLed(out port p, unsigned x)
{
  int i, j;
  timer t;
 
  for(i=0;i<INCREMENTS ;i++)
  {
    for(j=0;j<RESOLUTION ;j++)
    {  
      p <: x;
      wait(t, i*((TIMESTEP /RESOLUTION) /INCREMENTS ));
      p <: 0;
      wait(t, (INCREMENTS-i)*((TIMESTEP /RESOLUTION)/INCREMENTS)); 
    } 
  }
           
  //LED fade out
  for(i=1;i<INCREMENTS-10; i++)
  {
        
    for(j=0;j<RESOLUTION ;j++)
    {
      p <: x;
      wait(t, (INCREMENTS-i)*((TIMESTEP /RESOLUTION) /INCREMENTS));
      p <: 0;  
      wait(t, i*((TIMESTEP /RESOLUTION)/INCREMENTS  ));
     }
  }   
}

void doKeyLeds(chanend c)
{
  unsigned kledVal = 8; 
  unsigned kledValDir = 1;
  unsigned time, loopTerm = 1;
  timer t;
  
  while(loopTerm)
  { 
    t:> time;
    select
    {
      case c :> time:
      {
        if(time == CMD_KILL)
          loopTerm = 0;
        break;
      }
      case t when timerafter(time + 30000000) :> int tmpTime: 
      {
        fadeLed(p_kled, kledVal<<4);
        if(kledValDir)
        {
          kledVal >>= 1;  
          if(kledVal == 0)
          {
            kledVal = 2;
            kledValDir = !kledValDir;
          }    
        }
        else
        {
          kledVal <<= 1;  
        
          if(kledVal > 8) 
          {   
            kledVal = 4;
            kledValDir = !kledValDir;
          }
        }
        break;
      }
    }
  }
}

/**
 * @brief Watches buttons, sends kill cmd when buttons change to match killMask.  
 * @brief also responds to other threads requesting key values
 * @param c Chanend to thread that wants key data
 * @param killVal Mask of button values that will issue kill. e.g. if 0x1, button 1 is kill,
 * if 0xf any will issue kill
 * @return button value
 **/
unsigned doKeys(chanend c, unsigned killMask, in port p_key)
{
  unsigned newKeyVal,loopTerm = 1, oldKeyVal = 0xf;
  timer t; 
  wait(t, 100000); 
  // Wait for user to stop pressing keys

  p_key when pinseq(0xf) :> int _; 
 
  while(loopTerm)
  { 
    p_key when pinsneq(oldKeyVal) :> newKeyVal;
    oldKeyVal = newKeyVal;
    newKeyVal = (~newKeyVal);
      
    if((newKeyVal & killMask))  	// If killMask matched then send kill to thread
    {
      c <: CMD_KILL;
      loopTerm = 0;
    }
    else				// If not kill key(s) send to thread
    {
      c <: CMD_KEYVAL;
      c <: newKeyVal;
    }
    wait(t, 10000); 
  }
  //wait(t, 10000000);

  // Return button val when thread dies
  //printstrln("key led thread die");
  return newKeyVal;       
}

void doDemoLoop(in port p_key)
{
  chan c;
  chan c1;

  unsigned buttonVal;

  while(1)
  { 
    // Read button vals
    p_key when pinseq(0xf) :> int _;
    p_key :> buttonVal;
    //buttonVal >>= 4;

    // Wait for user press or update LEDs 
    par
    {
      doClockLedGlow(c, c1);                 // Do something pretty on 'clock' LEDs
      buttonVal = doKeys(c, 0xf, p_key) ;           // Watchs keys and does key leds (returns button no)
                                             // (Any key kills)
      doKeyLeds(c1);			       // Do something pretty on the 'key' LEDs
    }

    // Button Pressed, run relevant demo
    if((buttonVal & 1) == 1)
      runDemo(0, p_key);
    else if((buttonVal & 2) == 2)
      runDemo(1, p_key);
    else if((buttonVal & 4) == 4)
      runDemo(2, p_key);
    else if((buttonVal & 8) == 8)
      runDemo(3, p_key);
  }
}

void doVersionLeds()
{
  unsigned verNumMin = 1 << FIRMWARE_VER_MIN;
  unsigned verNumMaj = 1 << FIRMWARE_VER_MAJ;
  timer t;

  p_cled_g <: 1;
  p_cled_r <: 0;
  p_cled_0 <: >> verNumMin;
  p_cled_1 <: >> verNumMin;
  p_cled_2 <: >> verNumMin;
#if defined XC-1
  verNumMaj <<= 4;
#endif
  
  p_kled <: verNumMaj; 

  wait(t, 50000000);
  
  p_cled_g <: 0;
  p_cled_r <: 0;
  p_cled_0 <: 0;
  p_cled_1 <: 0;
  p_cled_2 <: 0;
  p_kled <: 0; 
}


int main2()
{
  unsigned tmp = 0;
  unsigned appnum = _GetNextAppNum();
  unsigned signature = appnum & 0xFFFF0000;
  unsigned char app1 = appnum >> 8;
  unsigned char app2 = appnum;
  int appnumok = (signature == SOFT_RESET_MAGIC_VALUE && app1 == app2);
 
  // Display version number on LEDs (in function so compiler frees timer)
  doVersionLeds(); 
  
#if   defined XC-1
  // If we are in JTAG mode then return for JTAG
  // If in SPI mode then go to SPI boot
  // If appnum bad go check for SPI/JTAG button holds else do normal demo boot 
  if(appnumok)
  {
    if((app1 == SOFT_RESET_BOOT_JTAG))
    { 
      return 0; 
    }
    if((app1 == SOFT_RESET_BOOT_SPI))
    {
      return 1;
    }
  }
#endif

  set_port_pull_up(p_key);
  
#if   defined XC-1
  p_key :> tmp;
  // If *A* key pressed, flash LEDS then return for JTAG
  if(tmp == 0xe)
  {
    // Set boot from JTAG
    SetBootSticky(SOFT_RESET_BOOT_JTAG);

    // Flash A LED to indicate going JTAG
    flashLed(p_kled, 10000000, 10000000, 3, 0x1);

    return 0;
  }
  
  // If *B* Key pressed flash LEDS then go to SPI
  if((tmp) == 0xd)    
  {
    // Set boot from SPI
    SetBootSticky(SOFT_RESET_BOOT_SPI);

    // Flash B LED to indicate going to SPI boot
    flashLed(p_kled, 10000000, 10000000, 3, 0x2);
 
    return 1; 
  }

  // Check boot config for spi mode 
  if(checkBootConfig_SPI())
  {
    return 1;  
  }

  // Clock down unused cores and interconnect
  setRegVal(1, 6, 2, 128);
  setRegVal(1, 6, 3, 128);
  setRegVal(0, 7, 0, 128);
  setRegVal(1, 6, 1, 128);
#endif


  // Run demo loop
  // Note: By putting this in a function we avoid having to free channels. yay.
  doDemoLoop(p_key);
 
  return 0;
}

int main(void)
{
 int spi =  main2();
 if(spi)
 {
   //#pragma stackcalls 20
   //_fini();
  bootFromSpi();
 }
   return 0;
}

